#!/usr/bin/python3
#
# Todo:
#	> use global vars for Col-indices. each function that changes em updates em
# 	> compat /w ecoTrack ? 1nt3gr4t3.

import sys, os
from datetime import datetime as dt

cutOff = True
ncut = 0
coolOff = 5

DELIM=","
OUT_FMT="%d-%m-%y %T"

def squash(data):

	print("squash plateaus")

	col_timestamp=0
	col_habitat=4
	col_x=6
	col_y=7
	col_tag=8

	omit_col_begin=4 #omit ts,ms,date,time
	
	nfast=0
	recent = []
	filtered = []
	for line in data:
		if line[0] == '#' or line[0] == '\n':
			if line.startswith("#timestamp"):
				global header
				header="#begin, end, dur, last-detect, SXE, PXE," + DELIM.join(line.split(',')[omit_col_begin:])

			continue

		l = line.split(DELIM)
		timestamp = int(l[col_timestamp])
		tag = l[ col_tag ]
		x = l[ col_x ]
		y = l[ col_y ]



		tagFound = False
		i=0
		while i < len(recent):

			samePos=(recent[i][5] == x and recent[i][6] == y)
			recentBeetle=recent[i][7] == tag
			fastBeetle = recentBeetle and not samePos
			ago = timestamp - recent[i][1]

			# print(i,":",recent[i])
			# print(f"tag: {tag}| timestamp: {timestamp}| ago:{ago}| coolOff:{coolOff}| fastBeetle:{fastBeetle}| {x},{y}:{recent[i][5]},{recent[i][6]}|recentBeetle:{recentBeetle}")
			if ago > coolOff or fastBeetle:
				if fastBeetle:
					nfast += 1

				## add new squashed / trackevent
				filtered.append(DELIM.join(map(str,recent[i])))

				del recent[i]	# del old
				continue		# dont incr idx!

			if recentBeetle:
				
				## update
				tagFound = True
				dur =  timestamp-recent[i][0]

				recent[i][1] = timestamp
				recent[i][2] = dur

			i +=1

		if not tagFound:

			## add recent
			recent.append([timestamp,timestamp,0]+l[omit_col_begin:])

	for r in recent:

		## cutOff
		if cutOff:
			global ncut
			ncut += r[2]
			r[1] = r[0]
			r[2] = 0

		filtered.append(DELIM.join(map(str,r)))

	print(f"	cutOff: {ncut}")
	print(f"	nfast: {nfast}")
	return filtered

def add_cols( data ):
	print("add cols")

	#after insertions
	col_start = 0
	col_end = 1
	col_lastdetect = 3
	col_sxe	= 4
	col_pxe	= 5
	col_habitat	= 7 
	col_x = 8
	col_y = 9

	known = {} #{tag: line}
	npxe = 0
	nsxe = 0

	for i,line in enumerate(data):

		l = line.split(DELIM)
		tag = l[7]

		if tag not in known:
			lastdetect = "NA"
			sxe = False
			pxe = False

		else:
			last_end = int(known[tag][col_end])
			this_start = int(l[col_start])
			lastdetect = this_start - last_end

			last_pos = known[tag][col_x], known[tag][col_y]
			this_pos = l[5], l[6]
			sxe = last_pos != this_pos
			if sxe:
				nsxe+=1

			last_habitat = known[tag][col_habitat]
			this_habitat = l[4]
			pxe = last_habitat != this_habitat and this_habitat != '0'
			if pxe:
				npxe+=1

		l.insert(col_lastdetect,lastdetect)
		l.insert(col_sxe, sxe)
		l.insert(col_pxe, pxe)

		known[tag] = l
		data[i] = DELIM.join(map(str,l))

	print(f"	nsxe: {nsxe}")
	print(f"	npxe: {npxe}")

	return data


def format(data):
	"sort, pad, human-readable times"
	print("format")

	#sort by end-time
	data.sort(key=lambda l: l.split(DELIM)[1])

	#TODO: get max len of each col and use that!
	# len(max(data, key=lambda l: len(l[s]))[s])
	for i,d in enumerate(data):
		d=d.split(DELIM)
		d[0]= dt.fromtimestamp(int( d[0] )).strftime(OUT_FMT) #convert timestamp to OUT_FMT
		d[1]= dt.fromtimestamp(int( d[1] )).strftime(OUT_FMT)
		# d[2]=d[2].rjust(4)	# dur
		# d[3]=d[3].rjust(6)	# lastdetect
		# d[4]=d[4].rjust(5)	# "FALSE"
		# d[5]=d[5].rjust(5)	# "FALSE"
		# d[6]=d[6].rjust(2)	# unit '13'
		# d[8]=d[8].rjust(1)	# x
		# d[9]=d[9].rjust(1)	# y
		# d[11]=d[11].rjust(2) # species
		# d[12]=d[12].rjust(5) # speciesnumber
		# d[13]=d[13].rjust(3) # weight
		# d[16]=d[16].rjust(4) # mlux
		data[i]=DELIM.join(map(str,d))

	return data

def write(outFile, data):

	outFile += f"-cool-{coolOff}-cut-{ncut}"

	print(f"writing to {outFile}")
	with open(outFile, "w") as f:
		f.writelines(header)
		f.writelines(data)

		cmd_compress=f"tar -zcvf {outFile}.tgz {outFile}" # ">/dev/null 2>/dev/null"
		os.system(cmd_compress)

def main(filename):

	with open(filename) as f:
		data = f.readlines()

	data = squash( data )
	data = add_cols( data )
	data = format( data )

	write(filename, data)

if __name__ == "__main__" :

	if len(sys.argv) == 1:
		this = os.path.basename(__file__)
		
		sys.exit(	f'Usage:	{this} dataFile coolOff cutOff\n\n'
					f'	another detection for a given tag and position\n'
					f'		* WITHIN coolOff seconds will be added to the last plateau\n'
					f'		* AFTER coolOff seconds will be used for a new plateau\n'
					f'		default: {coolOff}\n'
					f''
					f'	cutOff plateau which goes all the way to the end.\n'
					f'		eg beetle lost tag on sensor\n'
					f'		default: {cutOff}\n'
					f''
					f'	> squash plateaus (start/end/dur)\n'
					f'	> cutOff tags from end [optional]\n'
					f'	> add time-since-last-detection\n'
					f'	> add Patch Cross Event (PXE)\n'
					f'	> add Sensor Cross Event (SXE)\n'
					f'	> format: time,pad\n')

	if len(sys.argv) >= 3:
		coolOff = int(sys.argv[2])

	if len(sys.argv) >= 4:
		if sys.argv[3] in ('0','False'):
			cutOff = False

	main(sys.argv[1])

