#!/bin/bash
# # # #
# Determine time of last Trackevent in file/unit
#
# Why? Because timestamp is not human-readable and one unit can have multiple files.
#
# To use any function/alias from here source this file (source <this>)
##

function isNumber(){
	re='^[0-9]+$'
	if [[ "$1" =~ $re ]]; then
		return 1
	else
		return 0
	fi
}

function getLastDateUnit(){
	for i in "$@"; do
		echo "unit $i"
		files=`ls *unit-$i-*`
		if [ "$?" == 0 ]; then
			getLastDate $files
		fi
	done
}

function getLastDate() {
	# gets last line timestamp and transforms it to date
	# for more files, will get latest date of all files
	
	latest_date=0
	for file in "$@"; do
		timestamp=`tail -1 "$file" | cut -d';' -f1`
		isNumber "$timestamp"
		if [ "$?" == 0 ]; then
			continue
		fi

		# echo "${timestamp}; `date -d @"$timestamp" +%F-%T`; $file"
		if [ ${timestamp} -gt ${latest_date} ]; then
			latest_date="$timestamp"
			latest_file="$file"
		fi
	done

	last_date=`date -d @"$timestamp" +"%Y-%m-%d %T"` # 2>/dev/null #2>&1
	echo "${last_date}		${latest_file}"
	echo ""
}

alias getLastDateAllUnits="getLastDateUnit {2..13}"

# time must point to IP of backup-PI. (/etc/hosts or hostname)
alias getMostRecentBup="scp time:\`ssh time ls -rt /media/data/*gz | tail -1\` ."
